@@ -23,6 +23,7 @@
 #include "terminal-intl.h"
 #include <glade/glade.h>
 #include <libgnomeui/gnome-color-picker.h>
+#include <libgnomeui/gnome-font-picker.h>
 #include <libgnomeui/gnome-file-entry.h>
 #include <libgnomeui/gnome-icon-entry.h>
 #include <string.h>
@@ -126,6 +127,12 @@ static void       profile_editor_update_backspace_binding    (GtkWidget       *w
 static void       profile_editor_update_delete_binding       (GtkWidget       *widget,
                                                               TerminalProfile *profile);
 
+static void       profile_editor_update_use_theme_colors     (GtkWidget       *widget,
+                                                              TerminalProfile *profile);
+static void       profile_editor_update_use_system_font      (GtkWidget       *widget,
+                                                              TerminalProfile *profile);
+static void       profile_editor_update_font                 (GtkWidget       *widget,
+                                                              TerminalProfile *profile);
 
 
 static void profile_forgotten (TerminalProfile     *profile,
@@ -183,6 +190,44 @@ profile_editor_destroyed (GtkWidget       *editor,
   g_object_set_data (G_OBJECT (editor), "glade-xml", NULL);
 }
 
+static PangoFontDescription*
+fontpicker_get_desc (GtkWidget *font_picker)
+{
+  const char *current_name;
+  PangoFontDescription *current_desc;
+
+  current_name = gnome_font_picker_get_font_name (GNOME_FONT_PICKER (font_picker));
+  if (current_name)
+    current_desc = pango_font_description_from_string (current_name);
+  else
+    current_desc = NULL;
+
+  return current_desc;
+}
+
+static void
+fontpicker_set_if_changed (GtkWidget                  *font_picker,
+                           const PangoFontDescription *font_desc)
+{
+  PangoFontDescription *current_desc;
+
+  current_desc = fontpicker_get_desc (font_picker);
+
+  if (current_desc == NULL || !pango_font_description_equal (font_desc, current_desc))
+    {
+      char *str;
+
+      str = pango_font_description_to_string (font_desc);
+      gnome_font_picker_set_font_name (GNOME_FONT_PICKER (font_picker),
+                                       str);
+
+      g_free (str);
+    }
+
+  if (current_desc)
+    pango_font_description_free (current_desc);
+}
+
 /*
  * Profile callbacks
  */
@@ -286,6 +331,15 @@ profile_changed (TerminalProfile          *profile,
 
   if (mask & TERMINAL_SETTING_DELETE_BINDING)
     profile_editor_update_delete_binding (editor, profile);
+
+  if (mask & TERMINAL_SETTING_USE_THEME_COLORS)
+    profile_editor_update_use_theme_colors (editor, profile);
+    
+  if (mask & TERMINAL_SETTING_USE_SYSTEM_FONT)
+    profile_editor_update_use_system_font (editor, profile);
+
+  if (mask & TERMINAL_SETTING_FONT)
+    profile_editor_update_font (editor, profile);
   
   profile_editor_update_sensitivity (editor, profile);
 }
@@ -675,6 +729,48 @@ reset_compat_defaults_clicked (GtkWidget       *button,
   terminal_profile_reset_compat_defaults (profile);
 }
 
+static void
+use_theme_colors_toggled (GtkWidget       *checkbutton,
+                          TerminalProfile *profile)
+{
+  terminal_profile_set_use_theme_colors (profile,
+                                         gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (checkbutton)));
+}
+
+static void
+use_system_font_toggled (GtkWidget       *checkbutton,
+                         TerminalProfile *profile)
+{
+  terminal_profile_set_use_system_font (profile,
+                                        gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (checkbutton)));
+}
+
+static void
+font_set (GtkWidget       *fontpicker,
+          const char      *font_name,
+          TerminalProfile *profile)
+{
+  PangoFontDescription *desc;
+  PangoFontDescription *tmp;
+  
+  desc = pango_font_description_from_string (font_name);
+  if (desc == NULL)
+    {
+      g_warning ("Font name \"%s\" from font picker can't be parsed", font_name);
+      return;
+    }
+
+  /* Merge as paranoia against fontpicker giving us some junk */
+  tmp = pango_font_description_copy (terminal_profile_get_font (profile));
+  pango_font_description_merge (tmp, desc, TRUE);
+  pango_font_description_free (desc);
+  desc = tmp;
+  
+  terminal_profile_set_font (profile, desc);
+
+  pango_font_description_free (desc);
+}
+
 /*
  * initialize widgets
  */
@@ -811,6 +907,7 @@ terminal_profile_edit (TerminalProfile *profile,
       gtk_dialog_add_buttons (GTK_DIALOG (editor),
                               GTK_STOCK_CLOSE, GTK_RESPONSE_ACCEPT,
                               NULL);
+      gtk_dialog_set_default_response (GTK_DIALOG (editor), GTK_RESPONSE_ACCEPT);
       /* End "we have no Glade 2" workarounds */
 
 
@@ -1041,6 +1138,12 @@ terminal_profile_edit (TerminalProfile *profile,
       g_signal_connect (G_OBJECT (w), "changed",
                         G_CALLBACK (delete_binding_changed),
                         profile);
+
+      w = glade_xml_get_widget (xml, "use-theme-colors-checkbutton");
+      profile_editor_update_use_theme_colors (editor, profile);
+      g_signal_connect (G_OBJECT (w), "toggled",
+                        G_CALLBACK (use_theme_colors_toggled),
+                        profile);
       
       i = 0;
       while (i < TERMINAL_PALETTE_SIZE)
@@ -1066,26 +1169,45 @@ terminal_profile_edit (TerminalProfile *profile,
       profile_editor_update_palette (editor, profile);
 
       w = glade_xml_get_widget (xml, "font-hbox");
-  
-      fontsel = egg_xfont_selector_new ();
-      g_object_set_data (G_OBJECT (editor), "font-selector", fontsel);
- 
-      profile_editor_update_x_font (editor, profile);
-      
-      g_signal_connect (fontsel, "changed",
-			G_CALLBACK (font_changed), profile);
 
-      size_group = gtk_size_group_new (GTK_SIZE_GROUP_HORIZONTAL);
-      gtk_size_group_add_widget (size_group,
-                                 EGG_XFONT_SELECTOR (fontsel)->family_label);
-      gtk_size_group_add_widget (size_group,
-                                 glade_xml_get_widget (xml,
-                                                       "profile-name-label"));
-      gtk_size_group_add_widget (size_group,
-                                 glade_xml_get_widget (xml,
-                                                       "profile-icon-label"));
-      g_object_unref (G_OBJECT (size_group));
+      if (terminal_widget_supports_pango_fonts ())
+        {
+          fontsel = gnome_font_picker_new ();
+          g_object_set_data (G_OBJECT (editor), "font-selector", fontsel);
+
+          gnome_font_picker_set_title (GNOME_FONT_PICKER (fontsel),
+                                       _("Choose a terminal font"));
+          gnome_font_picker_set_mode (GNOME_FONT_PICKER (fontsel),
+                                      GNOME_FONT_PICKER_MODE_FONT_INFO);
+          gnome_font_picker_fi_set_show_size (GNOME_FONT_PICKER (fontsel), TRUE);
+
+          profile_editor_update_font (editor, profile);
+          g_signal_connect (G_OBJECT (fontsel), "font_set",
+                            G_CALLBACK (font_set),
+                            profile);
+        }
+      else
+        {
+          fontsel = egg_xfont_selector_new ();
+          g_object_set_data (G_OBJECT (editor), "font-selector", fontsel);
+ 
+          profile_editor_update_x_font (editor, profile);
       
+          g_signal_connect (fontsel, "changed",
+                            G_CALLBACK (font_changed), profile);
+
+          size_group = gtk_size_group_new (GTK_SIZE_GROUP_HORIZONTAL);
+          gtk_size_group_add_widget (size_group,
+                                     EGG_XFONT_SELECTOR (fontsel)->family_label);
+          gtk_size_group_add_widget (size_group,
+                                     glade_xml_get_widget (xml,
+                                                           "profile-name-label"));
+          gtk_size_group_add_widget (size_group,
+                                     glade_xml_get_widget (xml,
+                                                           "profile-icon-label"));
+          g_object_unref (G_OBJECT (size_group));
+        }
+          
       gtk_box_pack_start (GTK_BOX (w), GTK_WIDGET (fontsel), TRUE, TRUE, 0);
       gtk_widget_show_all (w);
       
@@ -1160,6 +1282,25 @@ profile_editor_update_sensitivity (GtkWidget       *editor,
       w = profile_editor_get_widget (editor, "scroll-background-checkbutton");
       gtk_widget_set_sensitive (w, FALSE);
     }
+
+  if (terminal_profile_get_use_theme_colors (profile))
+    {
+      set_insensitive (editor, "foreground-colorpicker", TRUE);
+      set_insensitive (editor, "background-colorpicker", TRUE);
+      set_insensitive (editor, "color-scheme-optionmenu", TRUE);
+    }
+  else
+    {      
+      set_insensitive (editor, "foreground-colorpicker",
+                       mask & TERMINAL_SETTING_FOREGROUND_COLOR);
+      
+      set_insensitive (editor, "background-colorpicker",
+                       mask & TERMINAL_SETTING_BACKGROUND_COLOR);
+      
+      set_insensitive (editor, "color-scheme-optionmenu",
+                       (mask & (TERMINAL_SETTING_BACKGROUND_COLOR |
+                                TERMINAL_SETTING_FOREGROUND_COLOR)));
+    }
   
 #if 0 /* uncomment once we've tested the sensitivity code */
   if (mask == last_mask)
@@ -1180,16 +1321,6 @@ profile_editor_update_sensitivity (GtkWidget       *editor,
 
   set_insensitive (editor, "show-menubar-checkbutton",
                    mask & TERMINAL_SETTING_DEFAULT_SHOW_MENUBAR);
-  
-  set_insensitive (editor, "foreground-colorpicker",
-                   mask & TERMINAL_SETTING_FOREGROUND_COLOR);
-
-  set_insensitive (editor, "background-colorpicker",
-                   mask & TERMINAL_SETTING_BACKGROUND_COLOR);
-
-  set_insensitive (editor, "color-scheme-optionmenu",
-                   (mask & (TERMINAL_SETTING_BACKGROUND_COLOR |
-                            TERMINAL_SETTING_FOREGROUND_COLOR)));
 
   set_insensitive (editor, "title-entry",
                    mask & TERMINAL_SETTING_TITLE);
@@ -1236,8 +1367,12 @@ profile_editor_update_sensitivity (GtkWidget       *editor,
   set_insensitive (editor, "palette-optionmenu",
                    mask & TERMINAL_SETTING_PALETTE);
 
-  set_insensitive (editor, "font-hbox",
-                   mask & TERMINAL_SETTING_X_FONT);
+  if (terminal_widget_supports_pango_fonts ())
+    set_insensitive (editor, "font-hbox",
+                     mask & TERMINAL_SETTING_FONT);
+  else    
+    set_insensitive (editor, "font-hbox",
+                     mask & TERMINAL_SETTING_X_FONT);
 
   set_insensitive (editor, "solid-radiobutton",
                    mask & TERMINAL_SETTING_BACKGROUND_TYPE);
@@ -1253,6 +1388,9 @@ profile_editor_update_sensitivity (GtkWidget       *editor,
                    mask & TERMINAL_SETTING_BACKSPACE_BINDING);
   set_insensitive (editor, "delete-binding-optionmenu",
                    mask & TERMINAL_SETTING_DELETE_BINDING);
+
+  set_insensitive (editor, "use-theme-colors-checkbutton",
+                   mask & TERMINAL_SETTING_USE_THEME_COLORS);
   
   {
     int i;
@@ -1648,6 +1786,9 @@ profile_editor_update_x_font (GtkWidget       *editor,
   char *weights[] = { "medium", "regular", NULL };
   gchar *name;
 
+  if (terminal_widget_supports_pango_fonts ())
+    return;
+  
   fontsel = g_object_get_data (G_OBJECT (editor), "font-selector");
 
   /* If the current selector font is the same as the new font,
@@ -1766,6 +1907,47 @@ profile_editor_update_delete_binding (GtkWidget       *editor,
                                terminal_profile_get_delete_binding (profile));
 }
 
+static void
+profile_editor_update_use_theme_colors (GtkWidget       *editor,
+                                        TerminalProfile *profile)
+{
+  GtkWidget *w;
+
+  w = profile_editor_get_widget (editor, "use-theme-colors-checkbutton");
+  
+  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (w),
+                                terminal_profile_get_use_theme_colors (profile));
+}
+
+static void
+profile_editor_update_use_system_font (GtkWidget       *editor,
+                                       TerminalProfile *profile)
+{
+  GtkWidget *w;
+
+#if 0
+  w = profile_editor_get_widget (editor, "use-system-font-checkbutton");
+  
+  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (w),
+                                terminal_profile_get_use_system_font (profile));
+#endif
+}
+
+static void
+profile_editor_update_font (GtkWidget       *editor,
+                            TerminalProfile *profile)
+{
+  GtkWidget *w;
+
+  if (!terminal_widget_supports_pango_fonts ())
+    return;
+  
+  w = g_object_get_data (G_OBJECT (editor), "font-selector");
+
+  fontpicker_set_if_changed (w,
+                             terminal_profile_get_font (profile));
+}
+
 static GtkWidget*
 profile_editor_get_widget (GtkWidget  *editor,
                            const char *widget_name)
